<?php

namespace Repository;

use Util\InstanceProvider;
use Teknisa\Model\Traits\TQuery;
use Util\Utilities;
use \Zeedhi\DevOps\Model\Projectmanagement\Entities;

class Attachments
{
    use TQuery;

    public function getEntityManager(){
        return InstanceProvider::getEntityManager();
    }

    public function getTaskAttachmentsExceptUploadedBy($issueId, $filter = null) {
        $fileServerUrl = \Util\Utilities::getFileServerFilesUrl();
        $query = <<<SQL
            SELECT '$fileServerUrl' || ATT.FILE_NAME AS FILENAME, ATT.ID, ATT.LABEL, TASK.NAME, 
                    ATT.TYPE_FILE AS FILE_TYPE, TO_CHAR(ATT.CREATED_AT, 'DD/MM/YYYY') AS CREATED_AT,
                    RES.NAME AS CREATED_BY, C.ID AS COMMENT_ID, 'I' AS CALL_TYPE,
                    TO_CHAR(ATT.CREATED_AT, 'YYYY/MM/DD') AS CREATED_AT_ORDER_BY
              FROM ATTACHMENT ATT
              JOIN RESOURCE_T RES ON (RES.ID = ATT.CREATED_BY)
         LEFT JOIN TASK           ON (TASK.ID = ATT.TASK)
         LEFT JOIN COMMENTS C     ON (ATT.COMMENTS = C.ID)
         LEFT JOIN TASK CTA       ON (CTA.ID = C.TASK)
             WHERE (ATT.TASK = :ID     OR C.TASK = :ID)
               AND (TASK.STATUS <> 'D' OR TASK.STATUS IS NULL)
               $filter
          ORDER BY ATT.CREATED_AT
SQL;

        $params = array (
            'ID' => $issueId
        );
        $orders = $this->getOrder(); 
        if (is_array($orders)) {
            foreach ($orders as $key => $order) {
                if($order == 'CREATED_AT.asc') {
                    $orders[$key] = 'CREATED_AT_ORDER_BY.asc';
                } else if ($order == 'CREATED_AT.desc') {
                    $orders[$key] = 'CREATED_AT_ORDER_BY.desc';
                }
            }
        }
        $finalQuery = $this->getFinalQuery($query, null, null, $orders);
        return $this->executeQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $finalQuery, $params);
    }

    public function findAllSpecification($issueId) {
        $fileServerUrl = \Util\Utilities::getFileServerFilesUrl();
        $query = <<<SQL
            SELECT '$fileServerUrl' || A.FILE_NAME AS FILENAME, T.ID AS TAST_ID, A.LABEL AS DOC_LABEL,A.ID,
                    A.ID ATTACHMENT_ID, C.DESCRIPTION  , 'I' AS CALL_TYPE,A.CREATED_AT,T.ID AS CALL_RELATIONSHIP
              FROM TASK T
              JOIN COMMENTS C
                ON C.TASK = T.ID
              JOIN COMMENT_TYPE Y
                ON Y.ID = C.COMMENT_TYPE_ID
               AND Y.CONTROL = 'S'
              JOIN ATTACHMENT A
                ON C.ID = A.COMMENTS
             WHERE T.STATUS = 'A'
               AND T.IS_ISSUE = 'Y' 
               AND T.ID = $issueId
          ORDER BY A.CREATED_AT DESC
SQL;

        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query);
    }

    public function getAllAttachmentsFolder($nrsoliserv,$basicIntranetFileDir, $cdClienteBase,
                                            $isTeknisaFunc,$filename = ''){
        $directorySeparator = DIRECTORY_SEPARATOR;

        $filter = '';

        $filterFolder = '';
        if($filename){
            $filterFolder = "AND $filename = TO_CHAR(LFT.DTALFASETRAM,'YYYYMMDDHH24MISS')";
        }

        $query = <<<SQL
			SELECT '$basicIntranetFileDir'||'$directorySeparator'||LFT.NRSOLISERV||'$directorySeparator'|| TO_CHAR(LFT.DTALFASETRAM,'YYYYMMDDHH24MISS')||'$directorySeparator' AS DIR,
			TO_CHAR(LFT.DTALFASETRAM,'YYYYMMDDHH24MISS') AS FILENAME
			FROM LOGFASETRAM LFT
                 LEFT JOIN OPERADOR OP  ON OP.CDOPERADOR  = LFT.CDOPERADOR
                 LEFT JOIN OPERADOR OPR ON OPR.CDOPERADOR = LFT.CDOPERRESP
                INNER JOIN SOLISERV SOL ON SOL.NRSOLISERV = :NRSOLISERV
                     WHERE LFT.NRSOLISERV = :NRSOLISERV
                     $filterFolder
                     $filter
SQL;
        $params = array(
            'NRSOLISERV' => $nrsoliserv,
            "cdClienteBase" => $cdClienteBase
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getAllSpecificationGEPFiles($nrsoliserv,$basicIntranetFileDir){
        $directorySeparator = DIRECTORY_SEPARATOR;
        $cdassuntoExpec = Utilities::getCdassuntoExpec();
        $query = <<<SQL
			  SELECT '$basicIntranetFileDir'||'$directorySeparator'||S.NRSOLISERV||'$directorySeparator'|| 'REQUISITO' ||'$directorySeparator' ||S.NRSOLISERV|| '_' ||I.CDITEMTRAB|| '.doc' AS FILENAME,
			                  '$basicIntranetFileDir'||'$directorySeparator'||S.NRSOLISERV||'$directorySeparator'|| 'REQUISITO' ||'$directorySeparator' ||S.NRSOLISERV|| '_' ||I.CDITEMTRAB|| '.docx' AS FILENAME_DOCX,
			          I.CDITEMTRAB,S.NRSOLISERV,S.NRSOLISERV|| '_' ||I.CDITEMTRAB|| '.doc' AS DOC_LABEL,B.NMITEMTRAB

                FROM TAREFA T
                JOIN SOLISERV S
                  ON S.CDFILIAL = T.CDFILIAL
                 AND S.NRSOLISERV = T.NRSOLISERV 
                JOIN ITEMTAREFA I ON (I.CDFILIAL = T.CDFILIAL AND I.NRTAREFA = T.NRTAREFA AND I.CDITEMTRAB IS NOT NULL)
                JOIN ITEMTRAB B ON (B.CDITEMTRAB = I.CDITEMTRAB)
               WHERE S.CDASSUNTO IN ('$cdassuntoExpec')
                 AND S.NRSOLISERV = :NRSOLISERV            
            GROUP BY S.NRSOLISERV, I.CDITEMTRAB, B.NMITEMTRAB, I.NRPRIOITTARE, I.NRSEQUITTARE
            ORDER BY I.NRPRIOITTARE, I.NRSEQUITTARE
SQL;
        $params = array(
            'NRSOLISERV' => $nrsoliserv,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getAttachmentTaskTracking($id, $conditions, $isIssue = false) {
        $queryNotIssue = '';
        if($isIssue){
            $queryNotIssue = <<<SQL

         UNION

            SELECT NVL(TASK.NAME,CTA.NAME) as TASK_NAME, ATT.FILE_NAME, ATT.ID, ATT.LABEL, TASK.NAME, ATT.TYPE_FILE, TO_CHAR(ATT.CREATED_AT, 'DD/MM/YYYY') AS CREATED_AT,
				   RES.NAME AS CREATED_BY, ATT.TASK
			  FROM ATTACHMENT ATT
			  JOIN RESOURCE_T RES ON (RES.ID = ATT.CREATED_BY)
	     LEFT JOIN TASK ON (TASK.ID = ATT.TASK)
		 LEFT JOIN COMMENTS C ON (ATT.COMMENTS = C.ID)
         LEFT JOIN TASK CTA ON (CTA.ID = C.TASK)
	         WHERE (ATT.TASK = :ID OR  C.TASK =:ID)
			   AND (TASK.STATUS <> 'D' OR TASK.STATUS IS NULL)
SQL;

        }

		$query =<<<SQL
        SELECT *
        FROM (
            SELECT NVL(TASK.NAME,CTA.NAME) as TASK_NAME, ATT.FILE_NAME, ATT.ID, ATT.LABEL,TASK.NAME, ATT.TYPE_FILE, TO_CHAR(ATT.CREATED_AT, 'DD/MM/YYYY') AS CREATED_AT,
				   RES.NAME AS CREATED_BY, ATT.TASK
			  FROM ATTACHMENT ATT
			  JOIN RESOURCE_T RES ON (RES.ID = ATT.CREATED_BY)
			  JOIN TASK_RELATIONSHIP	TAR ON (TAR.TASK_RELATIONSHIP = :ID AND TAR.TYPE = :TYPE_FIRST AND TAR.TASK = ATT.TASK )
		 LEFT JOIN COMMENTS C ON (ATT.COMMENTS = C.ID)
         LEFT JOIN TASK ON (TASK.ID = TAR.TASK)
         LEFT JOIN TASK CTA ON (CTA.ID = C.TASK)
	         WHERE (C.TASK = TAR.TASK OR TAR.TASK_RELATIONSHIP = :ID)
			   AND (TASK.STATUS <> 'D' OR TASK.STATUS IS NULL)

           $queryNotIssue
          )
	   $conditions
SQL;
		$params = array(
			'ID'  => $id,
            'TYPE_FIRST' => Entities\TaskRelationship::FIRST_TYPE,
		);

        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query, $params);
	}

}
