import { InstanceService } from '@/services/InstanceService';
import { AlertServiceCustom } from '@/services/AlertServiceCustom';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Http } from '@zeedhi/core';
import { LoadingService } from '@zeedhi/common';
import Messages from '@/services/Messages';

export class DMBudgetIssueController {
	public callRelationship = '';

	public callType = 'I';

	private validateEditBudget = true;

	public async beforeCancelGridBudgetItensIssue() {
		const budgetItensGridInstance = InstanceService.getInstanceTekGrid('gridbudgetItensIssue');
		await budgetItensGridInstance.datasource.get();
	}

	public async beforeCancelGridBudgetInstallmentsIssue() {
		const budgetItensGridInstance = InstanceService.getInstanceTekGrid('gridBudgetInstallmentsIssue');
		await budgetItensGridInstance.datasource.get();
	}

	public validateNumberParcel(parcel: number) {
		let validate = true;
		if (parcel >= 100) {
			AlertServiceCustom.customErrorAlert(Messages.ERR_MANY_PLOTS);
			validate = false;
		} else if (parcel <= 0) {
			AlertServiceCustom.customErrorAlert(Messages.ERR_ZERO_NUM_PLOTS);
			validate = false;
		} else if (!parcel) {
			AlertServiceCustom.customErrorAlert(Messages.ERR_INVALID_NUM_PLOTS);
			validate = false;
		}
		return validate;
	}

	public setDayPaymentDown(row: any, widgetPlots: any, prevRowDaysPayment: any, daysPaymentAdd: any) {
		let forPrevRowDaysPayment = prevRowDaysPayment;
		for (let x = parseInt(row.ORDER_PLOTS_NUM) - 2; x >= 0; x--) {
			const nextRow = widgetPlots.datasource.data[x];
			if (nextRow && nextRow.DAYS_PAYMENT >= forPrevRowDaysPayment) {
				nextRow.DAYS_PAYMENT = parseInt(nextRow.DAYS_PAYMENT) + parseInt(daysPaymentAdd);
				nextRow.DAYS_PAYMENT_COPY = nextRow.DAYS_PAYMENT;
				nextRow.DAYS_PAYMENT_inline = nextRow.DAYS_PAYMENT;
				nextRow.changed = true;
				forPrevRowDaysPayment = nextRow.DAYS_PAYMENT;
			}
		}
	}

	public setDayPaymentUp(row: any, widgetPlots: any, prevRowDaysPayment: any, daysPaymentAdd: any) {
		let forPrevRowDaysPayment = prevRowDaysPayment;
		for (let x = parseInt(row.ORDER_PLOTS_NUM); x <= widgetPlots.datasource.data.length; x++) {
			const nextRow = widgetPlots.datasource.data[x];
			if (nextRow && nextRow.DAYS_PAYMENT <= forPrevRowDaysPayment) {
				nextRow.DAYS_PAYMENT = parseInt(nextRow.DAYS_PAYMENT) + parseInt(daysPaymentAdd);
				nextRow.DAYS_PAYMENT_COPY = nextRow.DAYS_PAYMENT;
				nextRow.DAYS_PAYMENT_inline = nextRow.DAYS_PAYMENT;
				nextRow.changed = true;
				forPrevRowDaysPayment = nextRow.DAYS_PAYMENT;
			}
		}
	}

	public changeDayPaymentGridBudgetInstallmentsIssue(params: any) {
		const { row } = params;
		row.DAYS_PAYMENT_inline = params.component.value;
		row.DAYS_PAYMENT_COPY = row.DAYS_PAYMENT;
		row.DAYS_PAYMENT = params.component.value;
		const widgetPlots = InstanceService.getInstanceTekGrid('gridBudgetInstallmentsIssue');
		widgetPlots.datasource.data[row.ORDER_PLOTS_NUM - 1].DAYS_PAYMENT = row.DAYS_PAYMENT;
		const daysPaymentAdd = row.DAYS_PAYMENT_inline - row.DAYS_PAYMENT_COPY;
		row.DAYS_PAYMENT_COPY = row.DAYS_PAYMENT_inline;
		const prevRowDaysPayment = row.DAYS_PAYMENT_inline;
		if (daysPaymentAdd > 0) {
			this.setDayPaymentUp(row, widgetPlots, prevRowDaysPayment, daysPaymentAdd);
		} else {
			this.setDayPaymentDown(row, widgetPlots, prevRowDaysPayment, daysPaymentAdd);
		}
		this.updateGroupDataBudgetPLotsDayPayment(widgetPlots);
	}

	public setValuesAddParcel(parcel: any, widgetPlots: TekGrid) {
		const diff = parseFloat(parcel) - (widgetPlots.datasource.data.length) + 1;
		const valueTotal = Math.trunc(widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1].VALUE_PLOTS * 100) / 100;
		const value = Math.trunc((valueTotal / diff) * 100) / 100;
		const rest = valueTotal - ((value * 100 * diff) / 100);
		widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1].VALUE_PLOTS = value;
		widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1].changed = true;
		return [value, rest];
	}

	public removeParcel(widgetPlots: TekGrid) {
		const row = widgetPlots.datasource.data.pop();
		widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1]
			.VALUE_PLOTS = ((parseFloat(widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1]
			.VALUE_PLOTS) * 100) + parseFloat(row?.VALUE_PLOTS) * 100) / 100;
		widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1].changed = true;
	}

	public addParcel(widgetPlots: TekGrid, value: any, parcelLabel: string, parcel: number | null, rest: any) {
		const numParcel = widgetPlots.datasource.data.length + 1;
		const newRow = {
			VALUE_PLOTS: parseFloat(value),
			changed: true,
			ORDER_PLOTS: `${numParcel}° ${parcelLabel}`,
			ORDER_PLOTS_NUM: numParcel,
			DAYS_PAYMENT: parseInt(widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1].DAYS_PAYMENT, 10) + 30,
			BUDGET_ID: widgetPlots.datasource.data[widgetPlots.datasource.data.length - 1].BUDGET_ID,
		};
		if (parcel === widgetPlots.datasource.data.length) {
			newRow.VALUE_PLOTS += parseFloat(rest);
		}
		widgetPlots.datasource.data.push(newRow);
	}

	public async quantityPlotsChangeFormBudgetInstallmentsIssue() {
		const widgetPlots 	= InstanceService.getInstanceTekGrid('gridBudgetInstallmentsIssue');
		const formBudgetInstallmentsIssue = InstanceService.getInstanceZdForm('formBudgetInstallmentsIssue');
		const parcel = parseInt(formBudgetInstallmentsIssue.value.QUANTITY_PLOTS, 10);
		let value = null;
		let rest = null;
		const parcelLabel = 'Parcelas';
		if (this.validateNumberParcel(parcel)) {
			if (parcel === 1) {
				widgetPlots.datasource.data = [widgetPlots.datasource.data[0]];
				widgetPlots.datasource.data[0].VALUE_PLOTS = widgetPlots.datasource.data[0].PROPOSAL_VALUE;
			} else {
				if (parcel > widgetPlots.datasource.data.length) {
					const ret = this.setValuesAddParcel(parcel, widgetPlots);
					value = ret[0];
					rest = ret[1];
				}
				while (parcel !== widgetPlots.datasource.data.length) {
					if (parcel < widgetPlots.datasource.data.length) {
						this.removeParcel(widgetPlots);
					} else {
						this.addParcel(widgetPlots, value, parcelLabel, parcel, rest);
					}
				}
			}
			this.widgetPlotsUpdateGroups();
			widgetPlots.editing = true;
		}
	}

	public widgetPlotsUpdateGroups() {
		const widget = InstanceService.getInstanceTekGrid('gridBudgetInstallmentsIssue');
		const groupedData = widget.groupedData.filter((row) => {
			let filter = false;
			if (row.group) {
				filter = row.group;
			}
			return filter;
		});
		let total = 0;
		widget.datasource.data.forEach((row) => {
			groupedData.push(row);
			total += row.VALUE_PLOTS;
		});
		widget.groupedData.forEach((row) => {
			if (row.groupFooter) {
				row.VALUE_PLOTS = total;
				groupedData.push(row);
			}
		});
		widget.groupedData = groupedData;
	}

	public async beforeSaveGridBudgetItensIssue(params: any) {
		params.event?.preventDefault();
		LoadingService.show();
		try {
			const budgetItensGridInstance = InstanceService.getInstanceTekGrid('gridbudgetItensIssue');
			const formMaintenanceCostIssue = InstanceService.getInstanceZdForm('formMaintenanceCostIssue');
			await Http.post('/issueBudgetSave', {
				CALL_RELATIONSHIP: this.callRelationship,
				CALL_TYPE: this.callType,
				DATA: budgetItensGridInstance.groupedData,
				DATA_MAINTENANCE: formMaintenanceCostIssue.value,
			});
			AlertServiceCustom.customAlert(Messages.ALE_CHANGED_BUDGET);
			await budgetItensGridInstance.reload();
			await budgetItensGridInstance.cancelEditedRows();
			LoadingService.hide();
		} catch (e) {
			LoadingService.hide();
		}
	}

	public async beforeSaveGridBudgetInstallmentsIssue(params: any) {
		params.event?.preventDefault();
		const budgetItensGridInstance = InstanceService.getInstanceTekGrid('gridBudgetInstallmentsIssue');
		if (this.validateSaveWidgetPlots(budgetItensGridInstance)) {
			LoadingService.show();
			try {
				await Http.post('/issueBudgetInstallmentsSave', {
					CALL_RELATIONSHIP: this.callRelationship,
					DATA: budgetItensGridInstance.datasource.data,
				});
				await budgetItensGridInstance.reload();
				await budgetItensGridInstance.cancelEditedRows();
				LoadingService.hide();
			} catch (e) {
				LoadingService.hide();
			}
		}
	}

	public changeValuePlotsGridBudgetInstallmentsIssue(params: any) {
		const { row } = params;
		row.VALUE_PLOTS = params.component.value;
		const budgetItensGridInstance = InstanceService.getInstanceTekGrid('gridBudgetInstallmentsIssue');
		budgetItensGridInstance.datasource.data[row.ORDER_PLOTS_NUM - 1].VALUE_PLOTS = row.VALUE_PLOTS;
		this.updateGroupDataBudgetPLots(budgetItensGridInstance);
	}

	public async changeTabBudgetIssue(params: any) {
		try {
			this.callRelationship = params.component.events.callRelationship();
			const tabBudgetIssue = InstanceService.getInstanceTabs('tabBudgetIssue');
			const tab = tabBudgetIssue.tabs[tabBudgetIssue.activeTab];
			setTimeout(async () => {
				if (tab.events && tab.events.change && typeof tab.events.change === 'function') {
					await tab.events.change(params);
				}
			}, 100);
		} catch (e) {
			setTimeout(async () => {
				await this.changeTabBudgetIssue(params);
			}, 1000);
		}
	}

	public validadetotalparcel(widgetPlots: TekGrid) {
		let total = 0;
		const proposalValue = widgetPlots.datasource.data[0].PROPOSAL_VALUE;
		widgetPlots.datasource.data.forEach((row: any) => {
			if (typeof row.VALUE_PLOTS === 'string') {
				row.VALUE_PLOTS = row.VALUE_PLOTS.replace(/[.]/g, '').replace(',', '.');
			}
			total += row.VALUE_PLOTS * 100;
		});
		return total === proposalValue * 100;
	}

	public validateSaveWidgetPlots(widgetPlots: TekGrid) {
		let validate = true;
		if (!widgetPlots.datasource.data.length) {
			AlertServiceCustom.customErrorAlert(Messages.ERR_EMPTY_PLOTS);
			validate = false;
		} else if (!this.validadetotalparcel(widgetPlots)) {
			const mess = `${Messages.ERR_TOTAL_PLOTS} (${widgetPlots.datasource.data[0].PROPOSAL_VALUE_VIEW.trim()}).`;

			AlertServiceCustom.customErrorAlert(mess);
			validate = false;
		}
		return validate;
	}

	public async changeTabFrameBudgetInstallmentsIssue() {
		const gridBudgetInstallmentsIssue = InstanceService.getInstanceTekGrid('gridBudgetInstallmentsIssue');
		const formBudgetInstallmentsIssue = InstanceService.getInstanceZdForm('formBudgetInstallmentsIssue');
		await gridBudgetInstallmentsIssue.datasource.get();
		formBudgetInstallmentsIssue.value.QUANTITY_PLOTS = gridBudgetInstallmentsIssue.datasource.data.length;
		this.changeTabFrameBudgetItensIssue();
	}

	public async changeTabFrameBudgetItensIssue() {
		LoadingService.show();
		try {
			const budgetItensGridInstance = InstanceService.getInstanceTekGrid('gridbudgetItensIssue');
			const formMaintenanceCostIssue = InstanceService.getInstanceZdForm('formMaintenanceCostIssue');
			budgetItensGridInstance.columns.forEach((column: any) => {
				if (column.name === 'PROPOSAL_QUANTITY' && column.editable === false) {
					column.editable = true;
				}
			});
			formMaintenanceCostIssue.children.forEach((children: any) => {
				if (children.name === 'VALUE_DISCOUNT' && children.disabled === true) {
					children.disabled = false;
				}
				if (children.name === 'PERCENTAGE_DISCOUNT' && children.disabled === true) {
					children.disabled = false;
				}
			});
			formMaintenanceCostIssue.children.forEach((row) => {
				formMaintenanceCostIssue.value[row.name] = 0;
			});
			budgetItensGridInstance.datasource.get();
			const response = await Http.post('/maintenanceCost', {
				CALL_RELATIONSHIP: this.callRelationship,
				CALL_TYPE: this.callType,
			});
			formMaintenanceCostIssue.children.forEach((row) => {
				if (response && response.data && response.data.data && response.data.data[row.name]) {
					formMaintenanceCostIssue.value[row.name] = response.data.data[row.name];
				} else {
					formMaintenanceCostIssue.value[row.name] = 0;
				}
			});
			formMaintenanceCostIssue.value.changed = false;
			formMaintenanceCostIssue.value.ISSUE_ID = this.callRelationship;
			this.validateEditBudget = !!formMaintenanceCostIssue.value.VALIDATE_EDIT;
			if (!formMaintenanceCostIssue.value.VALIDATE_EDIT) {
				AlertServiceCustom.customErrorAlert(Messages.ALE_TASK_EDITING_BUDGE_WIDGET);
				budgetItensGridInstance.columns.forEach((column: any) => {
					if (column.name === 'PROPOSAL_QUANTITY' && column.editable === true) {
						column.editable = false;
					}
				});
				formMaintenanceCostIssue.children.forEach((children: any) => {
					if (children.name === 'VALUE_DISCOUNT' && children.disabled === false) {
						children.disabled = true;
					}
					if (children.name === 'PERCENTAGE_DISCOUNT' && children.disabled === false) {
						children.disabled = true;
					}
				});
			}
			LoadingService.hide();
		} catch (e) {
			LoadingService.hide();
		}
	}

	public formMaintenanceCostIssueChangeValueDiscount() {
		const formMaintenanceCostIssue = InstanceService.getInstanceZdForm('formMaintenanceCostIssue');
		let discountValue = formMaintenanceCostIssue.value.VALUE_DISCOUNT;
		const total = formMaintenanceCostIssue.value.MANUT_VALUE;
		if (parseFloat(discountValue) > parseFloat(total)) {
			discountValue = total;
		}
		const percent = parseFloat(((discountValue / total) * 100).toFixed(2));
		formMaintenanceCostIssue.value.TOTAL_VALUE = total - discountValue;
		formMaintenanceCostIssue.value.PERCENTAGE_DISCOUNT = percent;
		const widget = InstanceService.getInstanceTekGrid('gridbudgetItensIssue');
		widget.editing = true;
		formMaintenanceCostIssue.value.changed = true;
	}

	public formMaintenanceCostIssueChangePercentDiscount() {
		const formMaintenanceCostIssue = InstanceService.getInstanceZdForm('formMaintenanceCostIssue');
		let percentValue = formMaintenanceCostIssue.value.PERCENTAGE_DISCOUNT;
		const total = formMaintenanceCostIssue.value.MANUT_VALUE;
		if (percentValue > 100) {
			percentValue = 100;
			formMaintenanceCostIssue.value.PERCENTAGE_DISCOUNT = 100;
		}
		const discValue = (percentValue * total) / 100;
		formMaintenanceCostIssue.value.TOTAL_VALUE = total - discValue;
		formMaintenanceCostIssue.value.VALUE_DISCOUNT = discValue;
		const widget = InstanceService.getInstanceTekGrid('gridbudgetItensIssue');
		widget.editing = true;
		formMaintenanceCostIssue.value.changed = true;
	}

	public changeProposalQuantityGridBudgetItensIssue(params: any) {
		let proposalQuantity = params.component.value;
		if (!proposalQuantity) {
			proposalQuantity = 0;
		} else if (proposalQuantity < 0) {
			proposalQuantity *= -1;
		}
		const currentRow = params.row;
		currentRow.PROPOSAL_QUANTITY = proposalQuantity;
		const widget = InstanceService.getInstanceTekGrid('gridbudgetItensIssue');

		if (currentRow.BUDGET_ITEM_ID === '999999XXXX') {
			if (proposalQuantity > 100) {
				currentRow.PROPOSAL_QUANTITY = 100;
			}
		} else {
			currentRow.PROPOSAL_VALUE = proposalQuantity * currentRow.ITEM_VALUE_TRUE;
			this.updateGroupDataBudget(widget);
		}

		const indexSubTotalDescon = widget.groupedData.length - 2;
		const indexDescon = widget.groupedData.length - 3;
		const indexSubTotal = widget.groupedData.length - 5;
		const discountPercent = parseFloat(widget.groupedData[indexDescon].PROPOSAL_QUANTITY);
		const subTotalProposal = parseFloat(widget.groupedData[indexSubTotal].PROPOSAL_VALUE);
		const discountValue = Number(((subTotalProposal * discountPercent) / 100).toFixed(2)) * -1;
		widget.groupedData[indexSubTotalDescon].PROPOSAL_VALUE = discountValue;
		widget.groupedData[indexDescon].PROPOSAL_VALUE = Number(discountValue);

		this.updateGroupDataBudget(widget);
	}

	public updateGroupDataBudgetPLots(widget: TekGrid) {
		const indexTotal = widget.groupedData.length - 1;
		const indexSubTotal = widget.groupedData.length - 2;
		let total = 0;
		widget.groupedData.forEach((row, index) => {
			if (!row.group) {
				if (index !== indexSubTotal
					&& index !== indexTotal
				) {
					total += row.VALUE_PLOTS;
				} else {
					row.VALUE_PLOTS = total;
				}
			}
		});
	}

	public updateGroupDataBudgetPLotsDayPayment(widget: TekGrid) {
		widget.groupedData.forEach((row) => {
			if (!row.group && row.DAYS_PAYMENT && row.ORDER_PLOTS_NUM) {
				row.DAYS_PAYMENT = widget.datasource.data[row.ORDER_PLOTS_NUM - 1].DAYS_PAYMENT;
			}
		});
	}

	public updateGroupDataBudget(widget: TekGrid) {
		const indexTotal = widget.groupedData.length - 1;
		const indexSubTotalDescon = widget.groupedData.length - 2;
		const indexDescon = widget.groupedData.length - 3;
		const indexSubTotal = widget.groupedData.length - 5;
		let total = 0;
		widget.groupedData[0].changed = true;
		widget.groupedData.forEach((row, index) => {
			if (!row.group) {
				if (index !== indexSubTotal
					&& index !== indexSubTotalDescon
					&& index !== indexTotal
				) {
					total += row.PROPOSAL_VALUE;
				} else if (index === indexSubTotalDescon) {
					row.PROPOSAL_VALUE = widget.groupedData[indexDescon].PROPOSAL_VALUE;
				} else {
					row.PROPOSAL_VALUE = total;
				}
			}
		});
		const formMaintenanceCostIssue = InstanceService.getInstanceZdForm('formMaintenanceCostIssue');
		formMaintenanceCostIssue.value.MANUT_VALUE = Number(((total * formMaintenanceCostIssue.value.MAINTENANCE_PERCENTAGE) / 100).toFixed(2));
		this.formMaintenanceCostIssueChangePercentDiscount();
	}
}
